==> jboss-4.0.1-src.tar.bz2: jboss-4.0.1-src/webservice/src/main/org/jboss/webservice/encoding/ser/MetaDataTypeDesc.java <==
/**
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice.encoding.ser;

// $Id: MetaDataTypeDesc.java,v 1.6.2.5 2004/12/01 20:10:42 nihility Exp $

import org.apache.axis.description.AttributeDesc;
import org.apache.axis.description.ElementDesc;
import org.apache.axis.description.FieldDesc;
import org.apache.axis.description.TypeDesc;
import org.apache.axis.utils.BeanPropertyDescriptor;
import org.jboss.logging.Logger;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.webservice.deployment.BeanXMLMetaData;
import org.jboss.webservice.deployment.MetaDataRegistry;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

/**
 * An Axis TypeDesc that can get additional information from the ws4ee-deployment.xml
 * <p/>
 * Type <typeMapping> element has the same basic structure as defined by the wsdd schema.
 * <p/>
 * A additionally you can have a <typeDesc> child element which more information on every field
 * <p/>
 * Initially the TypeDesc is initalized through introspection from a java type, all fields will be encoded
 * xsd style in a random order.
 * <p/>
 * Here you can specify the name and encoding style for every element.
 * <p/>
 * The <elementOrder> is optional, but when present must contain all fields from the java type. This is
 * usefull when the schema complexType specifies <sequence> instead of <all>.
 * <p/>
 * Here is an example:
 * <p/>
 * <typeMapping
 *    qname='ns1:SequenceStruct2' xmlns:ns1='http://MarshallTest.org/xsd'
 *    type='java:org.jboss.ws4ee.test.marshalltest.rpcenc.SequenceStruct2'
 *    serializer='org.jboss.webservice.encoding.ser.MetaDataBeanSerializerFactory'
 *    deserializer='org.jboss.webservice.encoding.ser.MetaDataBeanDeserializerFactory'
 *    encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>
 *    <typeDesc>
 *       <elementDesc fieldName="varBase64Binary" xmlName="varBase64Binary" xmlType="xsd:base64Binary"/>
 *       <elementDesc fieldName="varHexBinary" xmlName="varHexBinary" xmlType="xsd:hexBinary"/>
 *       <elementDesc fieldName="varSoapString" xmlName="varSoapString" xmlType="soapenc:string"/>
 *       <elementDesc fieldName="varSoapBoolean" xmlName="varSoapBoolean" xmlType="soapenc:boolean"/>
 *       <elementDesc fieldName="varSoapFloat" xmlName="varSoapFloat" xmlType="soapenc:float"/>
 *       <elementDesc fieldName="varSoapDouble" xmlName="varSoapDouble" xmlType="soapenc:double"/>
 *       <elementDesc fieldName="varSoapDecimal" xmlName="varSoapDecimal" xmlType="soapenc:decimal"/>
 *       <elementDesc fieldName="varSoapInt" xmlName="varSoapInt" xmlType="soapenc:int"/>
 *       <elementDesc fieldName="varSoapShort" xmlName="varSoapShort" xmlType="soapenc:short"/>
 *       <elementDesc fieldName="varSoapByte" xmlName="varSoapByte" xmlType="soapenc:byte"/>
 *       <elementDesc fieldName="varSoapBase64" xmlName="varSoapBase64" xmlType="soapenc:base64"/>
 *       <elementDesc fieldName="varDateTimeArray" xmlName="varDateTimeArray" itemXmlType="xsd:dateTime"/>
 *       <elementOrder>
 *          <element name="varString"/>
 *          <element name="varInteger"/>
 *          <element name="varInt"/>
 *          <element name="varLong"/>
 *          <element name="varShort"/>
 *          <element name="varDecimal"/>
 *          <element name="varFloat"/>
 *          <element name="varDouble"/>
 *          <element name="varBoolean"/>
 *          <element name="varByte"/>
 *          <element name="varQName"/>
 *          <element name="varDateTime"/>
 *          <element name="varSoapString"/>
 *          <element name="varSoapBoolean"/>
 *          <element name="varSoapFloat"/>
 *          <element name="varSoapDouble"/>
 *          <element name="varSoapDecimal"/>
 *          <element name="varSoapInt"/>
 *          <element name="varSoapShort"/>
 *          <element name="varSoapByte"/>
 *          <element name="varBase64Binary"/>
 *          <element name="varHexBinary"/>
 *          <element name="varSoapBase64"/>
 *          <element name="varSequenceStruct"/>
 *          <element name="varDateTimeArray"/>
 *       </elementOrder>
 *    </typeDesc>
 * </typeMapping>
 *
 * @author thomas.diesler@jboss.org
 * @since 18-June-2004
 */
public class MetaDataTypeDesc extends TypeDesc
{
   // provide logging
   private final Logger log = Logger.getLogger(MetaDataTypeDesc.class);

   public MetaDataTypeDesc(Class javaClass, QName xmlType, BeanXMLMetaData metaData)
   {
      super(javaClass);

      if (javaClass == null)
         throw new IllegalArgumentException("javaClass cannot be null");
      if (xmlType == null)
         throw new IllegalArgumentException("xmlType cannot be null");

      // On the client side this should never be null because the ServiceImpl installs the typemappings.
      if (metaData == null)
         metaData = getServerSideMetaData(xmlType);

      try
      {
         if (metaData != null)
         {
            setXmlType(xmlType);
            setupElementDescriptions(metaData);
            setupElementOrder(metaData);
         }
      }
      catch (RuntimeException e)
      {
         log.error("Instanciation error", e);
         throw e;
      }
   }

   /**
    * The server side installs type mappings through the WSDD deployment, which constructs
    * the MetaDataBean[Serializer|Deserializer]Factory without metaData.
    *
    * This hack calls back to the AxisServer to obtain the missing metaData from a registry.
    * Ideally we would like to register the serializer/deserializer factories with axis complete
    * with meta data, but I could not figure out how to get to the TypeMappingRegistry at deploy time.
    */
   private BeanXMLMetaData getServerSideMetaData(QName xmlType)
   {
      BeanXMLMetaData metaData = null;
      try
      {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         ObjectName objectName = new ObjectName("jboss.ws4ee:service=AxisService");
         MetaDataRegistry registry = (MetaDataRegistry)server.getAttribute(objectName, "MetaDataRegistry");
         metaData = registry.getTypeMappingMetaData(xmlType);
      }
      catch (Exception e)
      {
         log.warn("Cannot obtain metaData from AxisService: " + e);
      }
      return metaData;
   }

   private void setupElementDescriptions(BeanXMLMetaData metaData)
   {
      // Setup element descriptions
      Iterator itElDesc = metaData.getElementMetaData();
      while (itElDesc.hasNext())
      {
         BeanXMLMetaData.ElementMetaData elMetaData = (BeanXMLMetaData.ElementMetaData)itElDesc.next();
         String fieldName = elMetaData.getFieldName();
         QName xmlName = elMetaData.getXmlName();
         QName xmlType = elMetaData.getXmlType();
         boolean asAttribute = elMetaData.isAsAttribute();
         boolean asContent = elMetaData.isAsContent();

         FieldDesc fieldDesc = null;
         if (asAttribute)
            fieldDesc = new AttributeDesc();
         else
         {
            fieldDesc = new ElementDesc();
            ((ElementDesc)fieldDesc).setAsContent(asContent);
            ((ElementDesc)fieldDesc).setItemXmlType(elMetaData.getItemXmlType());
         }

         fieldDesc.setFieldName(fieldName);
         fieldDesc.setXmlName(xmlName);
         fieldDesc.setXmlType(xmlType);
         addFieldDesc(fieldDesc);
      }
   }

   private void setupElementOrder(BeanXMLMetaData metaData)
   {
      // Setup element order
      Iterator itElementOrder = metaData.getElementOrder();
      if (itElementOrder.hasNext())
      {
         Map bpdMap = getPropertyDescriptorMap();
         BeanPropertyDescriptor[] bpdArr = new BeanPropertyDescriptor[bpdMap.size()];

         int index = 0;
         if (bpdMap.size() > 0)
         {
            ArrayList keys = new ArrayList(bpdMap.keySet());

            // The first one is fixed
            BeanPropertyDescriptor bpd = (BeanPropertyDescriptor)bpdMap.get("class");
            if (bpd != null)
            {
               bpdArr[index++] = bpd;
               keys.remove("class");
            }

            // copy the BeanPropertyDescriptors according to the meta data elementOrder
            while (itElementOrder.hasNext())
            {
               String fieldName = (String)itElementOrder.next();

               bpd = (BeanPropertyDescriptor)bpdMap.get(fieldName);
               if (bpd != null)
               {
                  bpdArr[index++] = bpd;
                  keys.remove(fieldName);
               }
            }

            // copy the remaining BeanPropertyDescriptors
            Iterator it = keys.iterator();
            while (it.hasNext())
            {
               String name = (String)it.next();
               bpd = (BeanPropertyDescriptor)bpdMap.get(name);
               if (bpd != null)
                  bpdArr[index++] = bpd;
            }

            setPropertyDescriptors(bpdArr);
         }
      }
   }
}

==> jboss-4.0.2-src.tar.bz2: jboss-4.0.2-src/webservice/src/main/org/jboss/webservice/encoding/ser/MetaDataTypeDesc.java <==
/**
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice.encoding.ser;

// $Id: MetaDataTypeDesc.java,v 1.6.2.6 2005/03/02 14:32:30 tdiesler Exp $

import org.jboss.axis.description.AttributeDesc;
import org.jboss.axis.description.ElementDesc;
import org.jboss.axis.description.FieldDesc;
import org.jboss.axis.description.TypeDesc;
import org.jboss.axis.utils.BeanPropertyDescriptor;
import org.jboss.logging.Logger;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.webservice.deployment.BeanXMLMetaData;
import org.jboss.webservice.deployment.MetaDataRegistry;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

/**
 * An Axis TypeDesc that can get additional information from the ws4ee-deployment.xml
 * <p/>
 * Type <typeMapping> element has the same basic structure as defined by the wsdd schema.
 * <p/>
 * A additionally you can have a <typeDesc> child element which more information on every field
 * <p/>
 * Initially the TypeDesc is initalized through introspection from a java type, all fields will be encoded
 * xsd style in a random order.
 * <p/>
 * Here you can specify the name and encoding style for every element.
 * <p/>
 * The <elementOrder> is optional, but when present must contain all fields from the java type. This is
 * usefull when the schema complexType specifies <sequence> instead of <all>.
 * <p/>
 * Here is an example:
 * <p/>
 * <typeMapping
 *    qname='ns1:SequenceStruct2' xmlns:ns1='http://MarshallTest.org/xsd'
 *    type='java:org.jboss.ws4ee.test.marshalltest.rpcenc.SequenceStruct2'
 *    serializer='org.jboss.webservice.encoding.ser.MetaDataBeanSerializerFactory'
 *    deserializer='org.jboss.webservice.encoding.ser.MetaDataBeanDeserializerFactory'
 *    encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>
 *    <typeDesc>
 *       <elementDesc fieldName="varBase64Binary" xmlName="varBase64Binary" xmlType="xsd:base64Binary"/>
 *       <elementDesc fieldName="varHexBinary" xmlName="varHexBinary" xmlType="xsd:hexBinary"/>
 *       <elementDesc fieldName="varSoapString" xmlName="varSoapString" xmlType="soapenc:string"/>
 *       <elementDesc fieldName="varSoapBoolean" xmlName="varSoapBoolean" xmlType="soapenc:boolean"/>
 *       <elementDesc fieldName="varSoapFloat" xmlName="varSoapFloat" xmlType="soapenc:float"/>
 *       <elementDesc fieldName="varSoapDouble" xmlName="varSoapDouble" xmlType="soapenc:double"/>
 *       <elementDesc fieldName="varSoapDecimal" xmlName="varSoapDecimal" xmlType="soapenc:decimal"/>
 *       <elementDesc fieldName="varSoapInt" xmlName="varSoapInt" xmlType="soapenc:int"/>
 *       <elementDesc fieldName="varSoapShort" xmlName="varSoapShort" xmlType="soapenc:short"/>
 *       <elementDesc fieldName="varSoapByte" xmlName="varSoapByte" xmlType="soapenc:byte"/>
 *       <elementDesc fieldName="varSoapBase64" xmlName="varSoapBase64" xmlType="soapenc:base64"/>
 *       <elementDesc fieldName="varDateTimeArray" xmlName="varDateTimeArray" itemXmlType="xsd:dateTime"/>
 *       <elementOrder>
 *          <element name="varString"/>
 *          <element name="varInteger"/>
 *          <element name="varInt"/>
 *          <element name="varLong"/>
 *          <element name="varShort"/>
 *          <element name="varDecimal"/>
 *          <element name="varFloat"/>
 *          <element name="varDouble"/>
 *          <element name="varBoolean"/>
 *          <element name="varByte"/>
 *          <element name="varQName"/>
 *          <element name="varDateTime"/>
 *          <element name="varSoapString"/>
 *          <element name="varSoapBoolean"/>
 *          <element name="varSoapFloat"/>
 *          <element name="varSoapDouble"/>
 *          <element name="varSoapDecimal"/>
 *          <element name="varSoapInt"/>
 *          <element name="varSoapShort"/>
 *          <element name="varSoapByte"/>
 *          <element name="varBase64Binary"/>
 *          <element name="varHexBinary"/>
 *          <element name="varSoapBase64"/>
 *          <element name="varSequenceStruct"/>
 *          <element name="varDateTimeArray"/>
 *       </elementOrder>
 *    </typeDesc>
 * </typeMapping>
 *
 * @author thomas.diesler@jboss.org
 * @since 18-June-2004
 */
public class MetaDataTypeDesc extends TypeDesc
{
   // provide logging
   private final Logger log = Logger.getLogger(MetaDataTypeDesc.class);

   public MetaDataTypeDesc(Class javaClass, QName xmlType, BeanXMLMetaData metaData)
   {
      super(javaClass);

      if (javaClass == null)
         throw new IllegalArgumentException("javaClass cannot be null");
      if (xmlType == null)
         throw new IllegalArgumentException("xmlType cannot be null");

      // On the client side this should never be null because the ServiceImpl installs the typemappings.
      if (metaData == null)
         metaData = getServerSideMetaData(xmlType);

      try
      {
         if (metaData != null)
         {
            setXmlType(xmlType);
            setupElementDescriptions(metaData);
            setupElementOrder(metaData);
         }
      }
      catch (RuntimeException e)
      {
         log.error("Instanciation error", e);
         throw e;
      }
   }

   /**
    * The server side installs type mappings through the WSDD deployment, which constructs
    * the MetaDataBean[Serializer|Deserializer]Factory without metaData.
    *
    * This hack calls back to the AxisServer to obtain the missing metaData from a registry.
    * Ideally we would like to register the serializer/deserializer factories with axis complete
    * with meta data, but I could not figure out how to get to the TypeMappingRegistry at deploy time.
    */
   private BeanXMLMetaData getServerSideMetaData(QName xmlType)
   {
      BeanXMLMetaData metaData = null;
      try
      {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         ObjectName objectName = new ObjectName("jboss.ws4ee:service=AxisService");
         MetaDataRegistry registry = (MetaDataRegistry)server.getAttribute(objectName, "MetaDataRegistry");
         metaData = registry.getTypeMappingMetaData(xmlType);
      }
      catch (Exception e)
      {
         log.warn("Cannot obtain metaData from AxisService: " + e);
      }
      return metaData;
   }

   private void setupElementDescriptions(BeanXMLMetaData metaData)
   {
      // Setup element descriptions
      Iterator itElDesc = metaData.getElementMetaData();
      while (itElDesc.hasNext())
      {
         BeanXMLMetaData.ElementMetaData elMetaData = (BeanXMLMetaData.ElementMetaData)itElDesc.next();
         String fieldName = elMetaData.getFieldName();
         QName xmlName = elMetaData.getXmlName();
         QName xmlType = elMetaData.getXmlType();
         boolean asAttribute = elMetaData.isAsAttribute();
         boolean asContent = elMetaData.isAsContent();

         FieldDesc fieldDesc = null;
         if (asAttribute)
            fieldDesc = new AttributeDesc();
         else
         {
            fieldDesc = new ElementDesc();
            ((ElementDesc)fieldDesc).setAsContent(asContent);
            ((ElementDesc)fieldDesc).setItemXmlType(elMetaData.getItemXmlType());
         }

         fieldDesc.setFieldName(fieldName);
         fieldDesc.setXmlName(xmlName);
         fieldDesc.setXmlType(xmlType);
         addFieldDesc(fieldDesc);
      }
   }

   private void setupElementOrder(BeanXMLMetaData metaData)
   {
      // Setup element order
      Iterator itElementOrder = metaData.getElementOrder();
      if (itElementOrder.hasNext())
      {
         Map bpdMap = getPropertyDescriptorMap();
         BeanPropertyDescriptor[] bpdArr = new BeanPropertyDescriptor[bpdMap.size()];

         int index = 0;
         if (bpdMap.size() > 0)
         {
            ArrayList keys = new ArrayList(bpdMap.keySet());

            // The first one is fixed
            BeanPropertyDescriptor bpd = (BeanPropertyDescriptor)bpdMap.get("class");
            if (bpd != null)
            {
               bpdArr[index++] = bpd;
               keys.remove("class");
            }

            // copy the BeanPropertyDescriptors according to the meta data elementOrder
            while (itElementOrder.hasNext())
            {
               String fieldName = (String)itElementOrder.next();

               bpd = (BeanPropertyDescriptor)bpdMap.get(fieldName);
               if (bpd != null)
               {
                  bpdArr[index++] = bpd;
                  keys.remove(fieldName);
               }
            }

            // copy the remaining BeanPropertyDescriptors
            Iterator it = keys.iterator();
            while (it.hasNext())
            {
               String name = (String)it.next();
               bpd = (BeanPropertyDescriptor)bpdMap.get(name);
               if (bpd != null)
                  bpdArr[index++] = bpd;
            }

            setPropertyDescriptors(bpdArr);
         }
      }
   }
}
